clear all;

addpath(genpath('c:\matlab'));
% This file compares the Wald test in the Hansen-Timmermann (2015) paper and the
% Clark-McCracken test, using DGPs from Clark and McCracken (JoE, 2005)

Nsim = 10000; %10,000 MCs
ss = 200; % simulation size (parts get dropped due to initial conditions)
alpha1 = 0.1;
beta1 = 0.8;
c_glob = 1;
c_loc = 3/sqrt(ss);
phi_auto = 0.5; 
burnin = 100;
ss = ss+burnin;

%paj = P/R
pajv = [0.2, 0.4, 0.6, 0.8, 1, 1.2, 1.4, 1.6, 1.8, 2.0];

for pajn=1:10;
paj = pajv(pajn); rho = 1/(1+paj); rhoj = rho;

for dgp=1:6;
    
for nsim = 1:Nsim;
   
xv = NaN(ss,1); yv = xv;
xv([1:2]) = zeros(2,1); yv([1:2]) = xv([1 2]);

if dgp==1; %homoskedastic, DGP1
    
for t = 1:ss;
    yv(t) = c_glob + randn(1,1);
end;

elseif dgp==2; %homoskedastic, DGP2
    
for t = 2:ss;
    u = randn(2,1);
    yv(t) = 0.3*yv(t-1) + c_loc*xv(t-1) + u(1);
    xv(t) = 0.5*xv(t-1) + u(2);
end;

elseif dgp==3; %heteroskedastic process, DGP 1
    
  Sig2u(1) = 1; Sig2u(2) = 1;
  u(2) = randn(1,1);
  omega0 = (1-alpha1-beta1);
               
for t = 2:ss;
    u(t) = sqrt(Sig2u(t-1))*randn(1,1);
    Sig2u(t)= omega0 + alpha1*u(t)^2 + beta1*Sig2u(t-1);
    yv(t) = c_glob + u(t);

end;
            
elseif dgp==4; %heteroskedastic, DGP 2
    Sig2u = zeros(2,2); u = randn(2,1);
    Sig2u(:,:,1) = eye(2); Sig2u(:,:,2) = eye(2); 
    u(:,2) = chol(Sig2u(:,:,2))'*randn(2,1);
             omega01 = Sig2u(1,1,1)*(1-alpha1-beta1);
             omega02 = Sig2u(2,2,1)*(1-alpha1-beta1);
             correl = 0;          
    
for t = 2:ss;
    u(:,t) = chol(Sig2u(:,:,t-1))'*randn(2,1);
    Sig2u(1,1,t)= omega01 + alpha1*u(1,t)^2 + beta1*Sig2u(1,1,t-1);
    Sig2u(2,2,t)= omega02 + alpha1*u(2,t)^2 + beta1*Sig2u(2,2,t-1);
    yv(t) = 0.3*yv(t-1) + c_loc*xv(t-1) + u(1,t);
    xv(t) = 0.5*xv(t-1) + u(2,t);
end;

elseif dgp==5; %homoskedastic with serial correlation, DGP 2
    
u = randn(2,1); siguf = sqrt(1-phi_auto^2);
for t = 2:ss;
    u(:,t) = phi_auto*u(:,t-1) + randn(2,1)/siguf;
    yv(t) = 0.3*yv(t-1) + c_loc*xv(t-1) + u(1,t);
    xv(t) = 0.5*xv(t-1) + u(2,t);
end;

else; %heteroskedastic with serial correlation, DGP 2
    Sig2eu = zeros(2,2); u = randn(2,1); siguf = sqrt(1-phi_auto^2);
    Sig2eu(:,:,1) = eye(2);
    eu(:,2) = chol(Sig2eu(:,:,1))'*randn(2,1);
             omega01 = Sig2eu(1,1,1)*(1-alpha1-beta1);
             omega02 = Sig2eu(2,2,1)*(1-alpha1-beta1);
      
for t = 2:ss;
    eu(:,t) = chol(Sig2eu(:,:,t-1))'*randn(2,1); 
    Sig2eu(1,1,t)= omega01 + alpha1*eu(1,t)^2 + beta1*Sig2eu(1,1,t-1);
    Sig2eu(2,2,t)= omega02 + alpha1*eu(2,t)^2 + beta1*Sig2eu(2,2,t-1);
    u(:,t) = phi_auto*u(:,t-1) + eu(:,t)/siguf;
    yv(t) = 0.3*yv(t-1) + c_loc*xv(t-1) + u(1,t);
    xv(t) = 0.5*xv(t-1) + u(2,t);
end;
    
end; % end dgp loop
    
y = yv(burnin+1:ss); x = xv(burnin+1:ss);
T = size(y,1);
R = ceil(T/(1+paj));
P = T - R;

for t = R:T-1;

    if dgp==1 | dgp==3;
        fore(t+1,1) = mean(y(1:t));
        fore0(t+1,1) = 0;
    else;
        xr = [y(1:t-1) x(1:t-1)]; xr0 = [y(1:t-1)];              
        yr = y(2:t);
        results = ols(yr,xr);
        fore(t+1,1) = [y(t) x(t)]*results.beta;
        results0 = ols(yr,xr0);
        fore0(t+1,1) = [y(t)]*results0.beta;
        
    end;
        
end;

if dgp==1 | dgp==3;
    beta2hat = mean(y(1:T));
    residsim = y(1:T) - beta2hat; sig2_eps_full = residsim'*residsim/rows(residsim);
else;
    results = ols(y(2:T),[y(1:T-1) x(1:T-1)]); beta2hat = results.beta(end);
    sig2_eps_full = results.sige;
end;

e = y(R+1:T) - fore(R+1:T);
e0 = y(R+1:T) - fore0(R+1:T);
sig2_eps = (e'*e)/rows(e);
difMSE(nsim,1) = (e0'*e0 - e'*e);

d = e0.*e0 - e.*e; dmean = d - mean(d); Omega = mean(dmean.*dmean); 
OOS_t(nsim,1) = sum(d)/sqrt(P*Omega);

if dgp==1 | dgp==3;
    xr = ones(T-1,1);
    Sn = y(2:T)'*xr*(inv(xr'*xr))*xr'*y(2:T);
    xr = ones(R-1,1);
    Snrho = y(2:R)'*xr*(inv(xr'*xr))*xr'*y(2:R);
    kappa = 1;
    difSn(nsim,1) = Sn - Snrho + kappa*log(rho);
else;
    xr = [y(1:T-1) x(1:T-1)]; xr0 = [y(1:T-1)];              
    S1 = y(2:T)'*xr0*(inv(xr0'*xr0))*xr0'*y(2:T);
    S2 = y(2:T)'*xr*(inv(xr'*xr))*xr'*y(2:T);
    xr = [y(1:R-1) x(1:R-1)]; xr0 = [y(1:R-1)];              
    S1rho = y(2:R)'*xr0*(inv(xr0'*xr0))*xr0'*y(2:R);
    S2rho = y(2:R)'*xr*(inv(xr'*xr))*xr'*y(2:R);
    kappa1 = cols(xr); kappa2 = cols(xr0);
    difSn(nsim,1) = -((S1 - S1rho + kappa1*log(rho)) - (S2 - S2rho + kappa2*log(rho)));

end;

end;

Ftests = [difMSE difSn];
c = corr([difMSE difSn]); [paj dgp c(1,2) mean([difMSE difSn])]

if pajn==5; testpairs(:,:,dgp) = Ftests; else; end;

critval_rec_t = xlsread('McCracken_Tables.xls','Table1','B1:N31'); 
critval_rec_F = xlsread('McCracken_Tables.xls','Table4','B1:N31'); 

i = find(critval_rec_F(:,1)==0.99);
OOS_t_rec_crit99 = [critval_rec_t(1,2:end); critval_rec_t(i,2:end)];
OOS_F_rec_crit99 = [critval_rec_F(1,2:end); critval_rec_F(i,2:end)];
i = find(critval_rec_F(:,1)==0.95);
OOS_t_rec_crit95 = [critval_rec_t(1,2:end); critval_rec_t(i,2:end)];
OOS_F_rec_crit95 = [critval_rec_F(1,2:end); critval_rec_F(i,2:end)];
i = find(critval_rec_F(:,1)==0.90);
OOS_t_rec_crit90 = [critval_rec_t(1,2:end); critval_rec_t(i,2:end)];
OOS_F_rec_crit90 = [critval_rec_F(1,2:end); critval_rec_F(i,2:end)];

OOS_t_critM99 = OOS_t_rec_crit99; OOS_t_critM95 = OOS_t_rec_crit95; OOS_t_critM90 = OOS_t_rec_crit90;
OOS_F_critM99 = OOS_F_rec_crit99; OOS_F_critM95 = OOS_F_rec_crit95; OOS_F_critM90 = OOS_F_rec_crit90;

ipaj = find(OOS_F_critM99(1,:)==paj);
OOS_F_crit99 = OOS_F_critM99(2:end,ipaj); OOS_F_crit95 = OOS_F_critM95(2:end,ipaj); 
OOS_F_crit90 = OOS_F_critM90(2:end,ipaj);
OOS_t_crit99 = OOS_t_critM99(2:end,ipaj); OOS_t_crit95 = OOS_t_critM95(2:end,ipaj); 
OOS_t_crit90 = OOS_t_critM90(2:end,ipaj);
Nreg = 1;

pval90t_out = zeros(nsim,1); pval95t_out = pval90t_out; pval99t_out = pval90t_out;
pval90F_out = zeros(nsim,2); pval95F_out = pval90F_out; pval99F_out = pval90F_out;

for j=1:Nsim;

if OOS_t(j) > OOS_t_crit90(Nreg); pval90t_out(j) = 1; else; end; 
if OOS_t(j) > OOS_t_crit95(Nreg); pval95t_out(j) = 1; else; end;
if OOS_t(j) > OOS_t_crit99(Nreg); pval99t_out(j) = 1; else; end;

if Ftests(j,1) > OOS_F_crit90(Nreg); pval90F_out(j,1) = 1; else; end; 
if Ftests(j,1) > OOS_F_crit95(Nreg); pval95F_out(j,1) = 1; else; end;
if Ftests(j,1) > OOS_F_crit99(Nreg); pval99F_out(j,1) = 1; else; end;
if Ftests(j,2) > OOS_F_crit90(Nreg); pval90F_out(j,2) = 1; else; end; 
if Ftests(j,2) > OOS_F_crit95(Nreg); pval95F_out(j,2) = 1; else; end;
if Ftests(j,2) > OOS_F_crit99(Nreg); pval99F_out(j,2) = 1; else; end;

end;

%rejection by Tn but not by Wald, rejection by Wald but not by Tn
npval90F = [pval90F_out(:,1)> pval90F_out(:,2) pval90F_out(:,2)> pval90F_out(:,1)];
npval95F = [pval95F_out(:,1)> pval95F_out(:,2) pval95F_out(:,2)> pval95F_out(:,1)];
npval99F = [pval99F_out(:,1)> pval99F_out(:,2) pval99F_out(:,2)> pval99F_out(:,1)];

cormean(pajn,:,dgp) = [dgp rhoj c(1,2) mean([difMSE difSn])]; 
rejt(pajn,:,dgp) = [rhoj mean([pval90t_out pval95t_out pval99t_out])];  
rejF(pajn,:,dgp) = [rhoj mean([pval90F_out pval95F_out pval99F_out])];
rejFn(pajn,:,dgp) = [rhoj mean([npval90F npval95F npval99F])];


end; %end of dgp loop

end; %end of paj loop

save Clark_McCracken200_power_Sn.mat;

figure(1);

subplot(2,3,1);
scatter(testpairs(:,1,1),testpairs(:,2,1));
title('dgp1');
xlabel('Tn'); ylabel('Wald');
subplot(2,3,2);
scatter(testpairs(:,1,2),testpairs(:,2,2));
title('dgp2');
xlabel('Tn'); ylabel('Wald');

subplot(2,3,3);
scatter(testpairs(:,1,3),testpairs(:,2,3));
title('dgp3');
xlabel('Tn'); ylabel('Wald');
subplot(2,3,4);
scatter(testpairs(:,1,4),testpairs(:,2,4));
title('dgp4');
xlabel('Tn'); ylabel('Wald');

subplot(2,3,5);
scatter(testpairs(:,1,5),testpairs(:,2,5));
title('dgp5');
xlabel('Tn'); ylabel('Wald');
subplot(2,3,6);
scatter(testpairs(:,1,6),testpairs(:,2,6));
title('dgp6');
xlabel('Tn'); ylabel('Wald');

print -depsc scatter500_Sn_power;


figure(2);

subplot(2,3,1);
qqplot(testpairs(:,1,1),testpairs(:,2,1));
title('dgp1');
xlabel('Tn'); ylabel('Wald');
subplot(2,3,2);
qqplot(testpairs(:,1,2),testpairs(:,2,2));
title('dgp2');
xlabel('Tn'); ylabel('Wald');

subplot(2,3,3);
qqplot(testpairs(:,1,3),testpairs(:,2,3));
title('dgp3');
xlabel('Tn'); ylabel('Wald');
subplot(2,3,4);
qqplot(testpairs(:,1,4),testpairs(:,2,4));
title('dgp4');
xlabel('Tn'); ylabel('Wald');

subplot(2,3,5);
qqplot(testpairs(:,1,5),testpairs(:,2,5));
title('dgp5');
xlabel('Tn'); ylabel('Wald');
subplot(2,3,6);
qqplot(testpairs(:,1,6),testpairs(:,2,6));
title('dgp6');
xlabel('Tn'); ylabel('Wald');

print -depsc qq500_Sn_power;

